@@ -54,9 +54,24 @@ SENDMAIL_BINARY = "/usr/sbin/sendmail"
 DISTRO_CODENAME = lsb_release.get_distro_information()['CODENAME']
 DISTRO_ID = lsb_release.get_distro_information()['ID']
 
+# progress information is written here
+PROGRESS_LOG="/var/run/unattended-upgrades.progress"
+
 # set from the sigint signal handler
 SIGNAL_STOP_REQUEST=False
 
+class LogInstallProgress(apt.progress.base.InstallProgress):
+    """ Install progress that writes to self.LOG
+        (/var/run/unattended-upgrades.progress by default)
+    """
+
+    LOG = PROGRESS_LOG 
+
+    def status_change(self, pkg, percent, status):
+        f=open(self.LOG, "w")
+        f.write(_("Progress: %s %% (%s)") % (percent, pkg))
+        f.close()
+        
 
 class Unlocked:
     """ context manager for unlocking the apt lock while cache.commit()
@@ -79,7 +94,7 @@ def is_dpkg_journal_dirty():
     (similar to debSystem::CheckUpdates)
     """
     d = os.path.dirname(
-        apt_pkg.Config.find_file("Dir::State::status"))+"/updates"
+        apt_pkg.config.find_file("Dir::State::status"))+"/updates"
     for f in os.listdir(d):
         if re.match("[0-9]+", f):
             return True
@@ -165,9 +180,10 @@ def match_whitelist_string(whitelist, origin):
 def upgrade_normal(cache, pkgs_to_upgrade, logfile_dpkg):
     error = None
     res = False
+    iprogress = LogInstallProgress()
     try:
         with Unlocked():
-            res = cache.commit()
+            res = cache.commit(install_progress=iprogress)
     except SystemError,e:
         error = e
     if res:
@@ -208,12 +224,21 @@ def upgrade_in_minimal_steps(cache, pkgs_to_upgrade, logfile_dpkg=""):
                 logging.debug("found  partition of size %s (%s)" % (len(changes), changes))
                 smallest_partition = changes
             cache.clear()
+
+        # write progress log information
+        f=open(PROGRESS_LOG+".minimal-steps", "w")
+        percent = len(to_upgrade)/len(pkgs_to_upgrade)*100.0
+        f.write(_("Progress: %s %%: (%s)") % (
+                    percent, ",".join(smallest_partition)))
+        f.close()
+
         # apply changes
         logging.debug("applying set %s" % smallest_partition)
         rewind_cache(cache, [cache[name] for name in smallest_partition])
         try:
+            iprogress = LogInstallProgress()
             with Unlocked():
-                res = cache.commit()
+                res = cache.commit(install_progress=iprogress)
                 if not res:
                     raise Exception("cache.commit() returned false")
             cache.open()
@@ -252,11 +277,19 @@ def check_changes_for_sanity(cache, allowed_origins, blacklist):
                 return False
             for blacklist_regexp in blacklist:
                 if re.match(blacklist_regexp, pkg.name):
-                    logging.debug("pkg '%s' blacklisted" % pkg.name)
+                    logging.debug("skipping blacklisted package '%s'" % pkg.name)
                     return False
             if pkg._pkg.selected_state == apt_pkg.SELSTATE_HOLD:
                 logging.debug("pkg '%s' is on hold" % pkg.name)
                 return False
+            # check if the package is unsafe to upgrade unattended
+            ignore_require_restart = apt_pkg.config.find_b(
+                "Unattended-Upgrade::IgnoreAppsRequireRestart", False)
+            if (pkg.marked_upgrade and
+                ignore_require_restart == False and
+                pkg.candidate.record.get("Upgrade-Requires") == "app-restart"):
+                logging.debug("pkg '%s' requires app-restart, not safe to upgrade unattended")
+                return False
     return True
 
 def pkgname_from_deb(debfile):
@@ -312,7 +345,7 @@ def conffile_prompt(destFile, prefix=""):
                     logging.debug("current md5: %s" % current_md5)
                     # hashes are the same, no conffile prompt
                     if current_md5 == md5:
-                        return False
+                        continue
                     # calculate md5sum from the deb (may take a bit)
                     dpkg_cmd = ["dpkg-deb","--fsys-tarfile",destFile]
                     tar_cmd = ["tar","-x","-O", "-f","-", "."+conf_file]
